"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const unified_1 = __importDefault(require("unified"));
const remark_parse_1 = __importDefault(require("remark-parse"));
const remark_rehype_1 = __importDefault(require("remark-rehype"));
const rehype_stringify_1 = __importDefault(require("rehype-stringify"));
const remark_gfm_1 = __importDefault(require("remark-gfm"));
const rehype_raw_1 = __importDefault(require("rehype-raw"));
const unist_util_visit_1 = __importDefault(require("unist-util-visit"));
const rehype_sanitize_1 = __importDefault(require("rehype-sanitize"));
const deepmerge_1 = __importDefault(require("deepmerge"));
const path_1 = __importDefault(require("path"));
const crypto_1 = __importDefault(require("crypto"));
const base64url_1 = __importDefault(require("base64url"));
const githubSanitizeSchema = require("hast-util-sanitize/lib/github.json");
const sanitizeSchema = deepmerge_1.default(githubSanitizeSchema, {
    attributes: {
        div: ["dataHfzSnippet"],
        img: ["dataSrc"],
        code: ["className"],
    },
});
const supportImgExts = [".jpg", ".jpeg", ".png", ".svg", ".gif"];
exports.default = (content, options) => {
    options.hash = options.hash || "content";
    return unified_1.default()
        .use(remark_parse_1.default)
        .use(remark_gfm_1.default)
        .use(remark_rehype_1.default, { allowDangerousHtml: true })
        .use(() => {
        return async (tree) => {
            const codeNodes = [];
            const imgNodes = [];
            unist_util_visit_1.default(tree, (node) => {
                if (node.tagName === "pre") {
                    codeNodes.push(node);
                }
                else if (node.tagName === "img") {
                    imgNodes.push(node);
                }
            });
            codeNodes.forEach((node) => {
                const codeElement = node.children[0];
                const { className } = codeElement.properties;
                if (!className ||
                    !["language-html", "language-hfz"].includes(className[0]) ||
                    codeElement.data?.meta) {
                    return;
                }
                const code = codeElement.children[0].value;
                node.tagName = "div";
                node.properties.dataHfzSnippet = encodeURIComponent(code);
                node.children = [];
            });
            await Promise.all(imgNodes.map(async (node) => {
                const src = node.properties.src;
                const imgDir = path_1.default.resolve(options?.outputPath, "imgs");
                if (src.startsWith("http")) {
                    node.properties.src = "";
                    node.properties.alt = "Unsupport Remote Image";
                    return;
                }
                const imgPath = path_1.default.resolve(options?.basePath, src);
                if (!fs_1.default.existsSync(imgPath)) {
                    node.properties.src = "";
                    node.properties.alt = "Image File Not Found";
                    return;
                }
                const imgExt = path_1.default.extname(src);
                if (!supportImgExts.includes(imgExt)) {
                    node.properties.src = "";
                    node.properties.alt = `Unsupport Image Type: ${imgExt}; support: ${supportImgExts.join(", ")}`;
                    return;
                }
                const imgStat = fs_1.default.statSync(imgPath);
                if (imgStat.size > 1024 * 1024 * 2) {
                    node.properties.src = "";
                    node.properties.alt = "Image File Too Large, limit 2M";
                    return;
                }
                let imgBuf;
                let hashContent;
                if (options.hash === "content") {
                    imgBuf = fs_1.default.readFileSync(imgPath);
                    hashContent = imgBuf;
                }
                else {
                    imgStat.size;
                    hashContent = `${src}_${+imgStat.mtime}`;
                }
                const imgId = base64url_1.default.encode(crypto_1.default.createHash("sha256").update(hashContent).digest());
                const imgName = imgId + imgExt;
                if (!fs_1.default.existsSync(path_1.default.join(imgDir, imgName))) {
                    if (!imgBuf)
                        imgBuf = fs_1.default.readFileSync(imgPath);
                    fs_1.default.mkdirSync(imgDir, { recursive: true });
                    fs_1.default.writeFileSync(path_1.default.join(imgDir, imgName), imgBuf);
                }
                delete node.properties.src;
                node.properties.dataSrc = imgName;
            }));
        };
    })
        .use(rehype_raw_1.default)
        .use(rehype_sanitize_1.default, sanitizeSchema)
        .use(rehype_stringify_1.default)
        .process(content)
        .then((value) => {
        fs_1.default.writeFileSync(path_1.default.join(options?.outputPath, "index.html"), value.toString());
    });
};
